@@ -13,13 +13,13 @@ import struct
 import subprocess
 from collections import namedtuple
 from contextlib import contextmanager
-from typing import Union
+from typing import Union, Iterator, Optional
 
 import pkg_resources
 import six
 import whichcraft
 from adbutils._utils import get_adb_exe
-from adbutils.errors import AdbError
+from adbutils.errors import AdbError, AdbTimeout
 from adbutils.mixin import ShellMixin
 from deprecation import deprecated
 
@@ -92,7 +92,7 @@ class _AdbStreamConnection(object):
         try:
             self.__conn = self._create_socket()
         except ConnectionRefusedError:
-            subprocess.run([adb_path(), "start-server"])
+            subprocess.run([adb_path(), "start-server"], timeout=20.0) # 20s should enough for adb start
             self.__conn = self._create_socket()
         return self
 
@@ -213,15 +213,20 @@ class AdbClient(object):
     def shell(self,
               serial: str,
               command: Union[str, list, tuple],
-              stream: bool = False) -> str:
+              stream: bool = False,
+              timeout: Optional[float] = None) -> str:
         """Run shell in android and return output
         Args:
             serial (str)
             command: list, tuple or str
             stream (bool): return stream instead of string output
+            timeout (float or None): only works when stream is False
 
         Returns:
             str or socket
+        
+        Raises:
+            AdbTimeout
         """
         assert isinstance(serial, six.string_types)
         if isinstance(command, (list, tuple)):
@@ -235,7 +240,13 @@ class AdbClient(object):
             c.check_okay()
             if stream:
                 return c
-            return c.read_until_close()
+            
+            # when no response in timeout, socket.timeout will raise
+            c.conn.settimeout(timeout)
+            try:
+                return c.read_until_close()
+            except socket.timeout:
+                raise AdbTimeout("shell exec timeout", "CMD={!r} TIMEOUT={:.1f}".format(command, timeout))
         except:
             if stream:
                 c.close()
@@ -244,15 +255,15 @@ class AdbClient(object):
             if not stream:
                 c.close()
     
-    def track_devices(self, limit_status=['device']):
+    def track_devices(self) -> Iterator[DeviceEvent]:
         """
-        track-devices
+        Report device state when changes
 
         Args:
             limit_status: eg, ['device', 'offline'], empty means all status
 
         Returns:
-            iter of DeviceEvent 
+            Iterator[DeviceEvent], DeviceEvent.status can be one of ['device', 'offline', 'unauthorized', 'absent']
         
         Raises:
             AdbError when adb-server was killed
@@ -264,26 +275,24 @@ class AdbClient(object):
             c.check_okay()
             while True:
                 output = c.read_string()
-                curr_devices = self._output2devices(output, limit_status)
+                curr_devices = self._output2devices(output)
                 for event in self._diff_devices(orig_devices, curr_devices):
                     yield event
                 orig_devices = curr_devices
 
-    def _output2devices(self, output: str, limit_status=[]):
+    def _output2devices(self, output: str):
         devices = []
         for line in output.splitlines():
             fields = line.strip().split("\t", maxsplit=1)
             if len(fields) != 2:
                 continue
             serial, status = fields
-            if limit_status and status not in limit_status:
-                continue
             devices.append(DeviceEvent(None, serial, status))
         return devices
 
     def _diff_devices(self, orig: list, curr: list):
         for d in set(orig).difference(curr):
-            yield DeviceEvent(False, d.serial, d.status)
+            yield DeviceEvent(False, d.serial, 'absent')
         for d in set(curr).difference(orig):
             yield DeviceEvent(True, d.serial, d.status)
 
@@ -421,16 +430,20 @@ class AdbDevice(ShellMixin):
     def shell(self,
               cmdargs: Union[str, list, tuple],
               stream: bool = False,
+              timeout: Optional[float] = None,
               rstrip=True) -> str:
         """Run shell inside device and get it's content
 
         Args:
             rstrip (bool): strip the last empty line (Default: True)
             stream (bool): return stream instead of string output (Default: False)
+            timeout (float): set shell timeout
 
         Returns:
             string of output
 
+        Raises:
+
         Examples:
             shell("ls -l")
             shell(["ls", "-l"])
@@ -438,7 +451,7 @@ class AdbDevice(ShellMixin):
         """
         if isinstance(cmdargs, (list, tuple)):
             cmdargs = subprocess.list2cmdline(cmdargs)
-        ret = self._client.shell(self._serial, cmdargs, stream=stream)
+        ret = self._client.shell(self._serial, cmdargs, stream=stream, timeout=timeout)
         if stream:
             return ret
         return ret.rstrip() if rstrip else ret
